unit MainUnit;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, Dbf, DB,
  Menus, ActnList, ExtCtrls, DBCtrls, DBGrids, StdCtrls, Printers, Buttons,
  AboutUnit, LR_Class, LR_DBSet, LR_Desgn, mrumanager, MRUList,
  newedit;

type
  TDataSetFirst = TAction;
  TDataSetPrior = TAction;
  TDataSetNext = TAction;
  TDataSetLast = TAction;
  TDataSetInsert = TAction;
  TDataSetDelete = TAction;
  TDataSetEdit = TAction;
  TDataSetPost = TAction;
  TDataSetCancel = TAction;
  TDataSetRefresh = TAction;

  { TContactForm }

  TContactForm = class(TForm)
    AClose: TAction;
    ACancel: TDataSetCancel;
    APrint: TAction;
    ARefresh: TDataSetRefresh;
    APost: TDataSetPost;
    AEdit: TDataSetEdit;
    ADelete: TDataSetDelete;
    AInsert: TDataSetInsert;
    ALast: TDataSetLast;
    ANext: TDataSetNext;
    APrior: TDataSetPrior;
    AFirst: TDataSetFirst;
    AQuit: TAction;
    AOpen: TAction;
    ANew: TAction;
    ActionList1: TActionList;
    DeleteButton: TButton;
    frDBDataSet1: TfrDBDataSet;
    ContactReport: TfrReport;
    EditButton: TButton;
    CurrentFileLabel: TLabel;
    SingleReport: TfrReport;
    InsertButton: TButton;
    ContactSource: TDatasource;
    Address1DBEdit: TDBEdit;
    Address2DBEdit: TDBEdit;
    Label15: TLabel;
    CurrentRecordLabel: TLabel;
    MRUManager1: TMRUManager;
    PreviousFileMenuItem: TMenuItem;
    MenuItem4: TMenuItem;
    PrintCurrentMenuItem: TMenuItem;
    MRUMenuManager1: TMRUMenuManager;
    NumOfRecordsLabel: TLabel;
    LastNameDBEdit: TDBEdit;
    TelephoneDBEdit: TDBEdit;
    FaxDBEdit: TDBEdit;
    MobileDBEdit: TDBEdit;
    EmailDBEdit: TDBEdit;
    TelephoneLabel: TLabel;
    FaxLabel: TLabel;
    MobileLabel: TLabel;
    WWWDBEdit: TDBEdit;
    EmailLabel: TLabel;
    WWWLabel: TLabel;
    TownDBEdit: TDBEdit;
    CountyDBEdit: TDBEdit;
    PostcodeDBEdit: TDBEdit;
    TitleDBEdit: TDBEdit;
    FirstDBEdit: TDBEdit;
    FirstNameLabel: TLabel;
    TitleLabel: TLabel;
    Address1Label: TLabel;
    Address2Label: TLabel;
    TownLabel: TLabel;
    CountyLabel: TLabel;
    PostcodeLabel: TLabel;
    MydbGrid: TdbGrid;
    CommentDBMemo: TDBMemo;
    DBNavigator1: TDBNavigator;
    LastNameLabel: TLabel;
    CommentsLabel: TLabel;
    MainMenu1: TMainMenu;
    FileMenu: TMenuItem;
    MenuItem1: TMenuItem;
    MenuItem2: TMenuItem;
    ExitMenuItem: TMenuItem;
    AboutMenuItem: TMenuItem;
    Panel1: TPanel;
    Panel2: TPanel;
    HelpMenu: TMenuItem;
    PrintAllMenuItem: TMenuItem;
    SaveMenuItem: TMenuItem;
    OpenMenuItem: TMenuItem;
    NewMenuItem: TMenuItem;
    MyDbf: TDbf;
    OpenDataDialog: TOpenDialog;
    SaveDataDialog: TSaveDialog;
    procedure AboutMenuItemClick(Sender: TObject);
    procedure ANewExecute(Sender: TObject);
    procedure ACloseExecute(Sender: TObject);
    procedure AQuitExecute(Sender: TObject);
    procedure ACloseUpdate(Sender: TObject);
    procedure AOpenExecute(Sender: TObject);
    procedure DataOpen(Sender: TObject);
    procedure DBNavigator1Click(Sender: TObject; Button: TDBNavButtonType);
    procedure DeleteButtonClick(Sender: TObject);
    procedure DoDataAction(Sender: TObject);
    procedure EditButtonClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
    procedure FormCreate(Sender: TObject);
    procedure HaveDataNotEmpty(Sender: TObject);
    procedure InEditMode(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure InsertButtonClick(Sender: TObject);
    procedure MydbGridCellClick(Column: TColumn);
    procedure PrintAllMenuItemClick(Sender: TObject);
    procedure PrintCurrentMenuItemClick(Sender: TObject);
    procedure SingleReportGetValue(const ParName: string; var ParValue: variant);
    procedure MRUMenuManager1RecentFile(Sender: TObject; const AFileName: string);
  private
    { private declarations }
    procedure CreateTable(AFileName: string);
    procedure OpenTable(AFileName: string);
    procedure CloseTable;
    procedure PrintWholeFile;
    procedure PrintCurrentRecord;
    procedure RecordForm(AChoice: integer);
    procedure EditFormData;
    procedure NewRecData;
  public
    { public declarations }
  end;

var
  ContactForm: TContactForm;

implementation

resourcestring
  SSaveData = 'There is still unsaved data. Save now ?';

{ANewExecute creates a new table (FileName) and enables the Print menu item}
procedure TContactForm.ANewExecute(Sender: TObject);
begin
  with SaveDataDialog do
    if Execute then
    begin
      CreateTable(FileName);
      PrintAllMenuItem.Enabled := True;
    end;
end;

{Shows the AboutBox}
procedure TContactForm.AboutMenuItemClick(Sender: TObject);
begin
  AboutForm.ShowModal;
end;

{Closes an open database and disables the Print Menu item}
procedure TContactForm.ACloseExecute(Sender: TObject);
begin
  CloseTable;
  CommentDBMemo.Clear;
  NumOfRecordsLabel.Caption := 'Number of Records : 0';
  CurrentRecordLabel.Caption := 'Current Record : 0';
  CurrentFileLabel.Caption := 'Current File : NONE';
  InsertButton.Enabled := False;
  EditButton.Enabled := False;
  DeleteButton.Enabled := False;
  PrintAllMenuItem.Enabled := False;
  PrintCurrentMenuItem.Enabled := False;
  SaveMenuItem.Enabled := False;
end;

procedure TContactForm.ACloseUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled := not (MyDbf.State in dsEditModes);
end;

{Takes the file (FileName) and passes it to the OpenTable procedure
 to open the Contacts database}
procedure TContactForm.AOpenExecute(Sender: TObject);
begin
  with OpenDataDialog do
    if Execute then
      OpenTable(FileName);
end;

{Opens an existing Contacts database and enables the Print menu item}
procedure TContactForm.OpenTable(AFileName: string);
begin
  with MyDbf do
  begin
    Close;
    TableName := AFileName;
    Exclusive := True;
    Open;
  end;
  NumOfRecordsLabel.Caption := 'Number of Records : ' + IntToStr(MyDbf.RecordCount);
  CurrentRecordLabel.Caption := 'Current Record : ' + IntToStr(MyDbf.PhysicalRecNo);
  CurrentFileLabel.Caption := 'Current File : ' + AFileName;
  PrintAllMenuItem.Enabled := True;
  PrintCurrentMenuItem.Enabled := True;
  SaveMenuItem.Enabled := True;
  InsertButton.Enabled := True;
  EditButton.Enabled := True;
  DeleteButton.Enabled := True;
  MRUMenuManager1.AddToRecent(AFileName);
end;


{Creates a new Contacts database with the name in (AFileName)}
procedure TContactForm.CreateTable(AFileName: string);
begin
  with MyDbf do
  begin
    Close;
    with FieldDefs do
    begin
      Clear;
      Add('Title', ftString, 10);
      Add('FirstName', ftString, 30);
      Add('LastName', ftString, 30);
      Add('Address1', ftString, 50);
      Add('Address2', ftString, 50);
      Add('Town', ftString, 30);
      Add('County', ftString, 30);
      Add('PostCode', ftString, 20);
      Add('Telephone', ftString, 15);
      Add('Fax', ftString, 15);
      Add('Mobile', ftString, 15);
      Add('Email', ftString, 40);
      Add('WWW', ftString, 40);
      Add('Comments', ftMemo, 0);
    end;
    TableName := AFileName;
    CreateTable;
    Exclusive := True;
    Open;
    AddIndex('LastName', 'LastName', []);
    InsertButton.Enabled := True;
    EditButton.Enabled := True;
    DeleteButton.Enabled := True;
     SaveMenuItem.Enabled := True;
  end;
end;

{Closes an open table}
procedure TContactForm.CloseTable;
begin
  MyDbf.Close;
end;

{Exits the application}
procedure TContactForm.AQuitExecute(Sender: TObject);
begin
  Close;
end;

procedure TContactForm.DataOpen(Sender: TObject);
begin
  with MyDbf do
    (Sender as TAction).Enabled := Active;

  NumOfRecordsLabel.Caption := 'Number of Records : ' + IntToStr(MyDbf.RecordCount);
  CurrentRecordLabel.Caption := 'Current Record : ' + IntToStr(MyDbf.PhysicalRecNo);
end;

procedure TContactForm.DBNavigator1Click(Sender: TObject; Button: TDBNavButtonType);
begin
  case Button of
    nbFirst, nbNext, nbPrior, nbLast: CurrentRecordLabel.Caption :=
        'Current Record : ' + IntToStr(MyDbf.PhysicalRecNo);
  end;
end;

procedure TContactForm.DeleteButtonClick(Sender: TObject);
begin
  if MessageDlg('Do you really want to delete this entry?', mtWarning,
    [mbYes, mbNo], 0) = mrYes then
  begin
    MyDbf.Edit;
    MyDbf.Delete;
    MyDbf.PackTable;
    MyDbf.Refresh;
  end;
  NumOfRecordsLabel.Caption := 'Number of Records : ' + IntToStr(MyDbf.RecordCount);
  CurrentRecordLabel.Caption := 'Current Record : ' + IntToStr(MyDbf.PhysicalRecNo);
end;

procedure TContactForm.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  CloseTable;
end;

procedure TContactForm.InsertButtonClick(Sender: TObject);
begin
  RecordForm(1);
  if NewEditForm.ModalResult = mrOk then
    NewRecData;

  NumOfRecordsLabel.Caption := 'Number of Records : ' + IntToStr(MyDbf.RecordCount);
  CurrentRecordLabel.Caption := 'Current Record : ' + IntToStr(MyDbf.PhysicalRecNo);
end;

procedure TContactForm.MydbGridCellClick(Column: TColumn);
begin
  CurrentRecordLabel.Caption := 'Current Record : ' + IntToStr(MyDbf.PhysicalRecNo);
end;


procedure TContactForm.MRUMenuManager1RecentFile(Sender: TObject;
  const AFileName: string);
begin
  OpenTable(AFileName);
end;



{Print the whole database}
{A preview report sheet is shown and the user can print it from there.}
procedure TContactForm.PrintAllMenuItemClick(Sender: TObject);
begin
  PrintWholeFile;
end;

{The current record is printed on a single sheet of paper.}
procedure TContactForm.PrintCurrentMenuItemClick(Sender: TObject);
begin
  PrintCurrentRecord;
end;

procedure TContactForm.SingleReportGetValue(const ParName: string;
  var ParValue: variant);
begin
  if ParName = 'title' then
    ParValue := TitleDBEdit.Text;
  if ParName = 'first' then
    ParValue := FirstDBEdit.Text;
  if ParName = 'last' then
    ParValue := LastNameDBEdit.Text;
  if ParName = 'addr1' then
    ParValue := Address1DBEdit.Text;
  if ParName = 'addr2' then
    ParValue := Address2DBEdit.Text;
  if ParName = 'town' then
    ParValue := TownDBEdit.Text;
  if ParName = 'county' then
    ParValue := CountyDBEdit.Text;
  if ParName = 'postcode' then
    ParValue := PostcodeDBEdit.Text;
  if ParName = 'phone' then
    ParValue := TelephoneDBEdit.Text;
  if ParName = 'mobile' then
    ParValue := MobileDBEdit.Text;
  if ParName = 'fax' then
    ParValue := FaxDBEdit.Text;
  if ParName = 'email' then
    ParValue := EmailDBEdit.Text;
  if ParName = 'www' then
    ParValue := WWWDBEdit.Text;
  if ParName = 'notes' then
    ParValue := CommentDBMemo.Text;
end;

procedure TContactForm.DoDataAction(Sender: TObject);
begin
  case (Sender as TAction).Tag of
    0: MyDbf.First;
    1: MyDbf.Prior;
    2: MyDbf.Next;
    3: MyDbf.Last;
    4: MyDbf.Insert;
    5: MyDbf.Delete;
    6: MyDbf.Edit;
  end;
  NumOfRecordsLabel.Caption := 'Number of Records : ' + IntToStr(MyDbf.RecordCount);
  CurrentRecordLabel.Caption := 'Current Record : ' + IntToStr(MyDbf.PhysicalRecNo);
end;

procedure TContactForm.EditButtonClick(Sender: TObject);
begin
  RecordForm(2);


  EditFormData;

end;

procedure TContactForm.FormCloseQuery(Sender: TObject; var CanClose: boolean);
begin
  CanClose := not (MyDbf.State in dsEditModes);
  if not CanClose then
    case MessageDlg(SSaveData, mtWarning, [mbYes, mbNo, mbCancel], 0) of
      mrYes:
      begin
        MyDbf.Post;
        CanClose := True;
      end;
      mrNo: CanClose := True;
    end;
  MRUMenuManager1.SaveRecentFilesToIni;
end;

procedure TContactForm.FormCreate(Sender: TObject);
begin
  CommentDBMemo.Clear;
  MRUMenuManager1.LoadRecentFilesFromIni;
end;

procedure TContactForm.HaveDataNotEmpty(Sender: TObject);
begin
  with MyDbf do
    (Sender as TAction).Enabled := Active and not (EOF and BOF);
end;

procedure TContactForm.InEditMode(Sender: TObject);
begin
  with MyDbf do
    (Sender as TAction).Enabled := State in dsEditModes;

end;

{Previews the entire database  and allows the user to print from there.
 This uses the LazReport components.}
procedure TContactForm.PrintWholeFile;
begin
  ContactReport.LoadFromFile(ExtractFilePath(ParamStr(0)) + 'contact.lrf');
  ContactReport.ShowReport;
end;

{Prints the current record on 1 sheet of paper}
procedure TContactForm.PrintCurrentRecord;
begin
  SingleReport.LoadFromFile(ExtractFilePath(ParamStr(0)) + 'single.lrf');
  SingleReport.ShowReport;
end;

procedure TContactForm.RecordForm(AChoice: integer);
begin
  case AChoice of
    1:
    begin
      NewEditForm.Caption := 'New Record';
    end;
    2:
    begin
      NewEditForm.Caption := 'Edit Record';
      NewEditForm.TitleEdit.Text := ContactForm.TitleDBEdit.Text;
      NewEditForm.FirstNameEdit.Text := ContactForm.FirstDBEdit.Text;
      NewEditForm.LastNameEdit.Text := ContactForm.LastNameDBEdit.Text;
      NewEditForm.Address1Edit.Text := ContactForm.Address1DBEdit.Text;
      NewEditForm.Address2Edit.Text := ContactForm.Address2DBEdit.Text;
      NewEditForm.TownEdit.Text := ContactForm.TownDBEdit.Text;
      NewEditForm.CountyEdit.Text := ContactForm.CountyDBEdit.Text;
      NewEditForm.PostCodeEdit.Text := ContactForm.PostCodeDBEdit.Text;
      NewEditForm.TelephoneEdit.Text := ContactForm.TelephoneDBEdit.Text;
      NewEditForm.MobileEdit.Text := ContactForm.MobileDBEdit.Text;
      NewEditForm.FaxEdit.Text := ContactForm.FaxDBEdit.Text;
      NewEditForm.EmailEdit.Text := ContactForm.EmailDBEdit.Text;
      NewEditForm.WebSiteEdit.Text := ContactForm.WWWDBEdit.Text;
      NewEditForm.CommentsMemo.Text := ContactForm.CommentDBMemo.Text;
    end;
  end;
  NewEditForm.ShowModal;
end;

procedure TContactForm.EditFormData;
begin
  MyDBF.Edit;
  MyDBF.FieldByName('Title').AsString := NewEditForm.TitleEdit.Text;
  MyDBF.FieldByName('FirstName').AsString := NewEditForm.FirstNameEdit.Text;
  MyDBF.FieldByName('LastName').AsString := NewEditForm.LastNameEdit.Text;
  MyDBF.FieldByName('Address1').AsString := NewEditForm.Address1Edit.Text;
  MyDBF.FieldByName('Address2').AsString := NewEditform.Address2Edit.Text;
  MyDBF.FieldByName('Town').AsString := NewEditForm.TownEdit.Text;
  MyDBF.FieldByName('County').AsString := NewEditForm.CountyEdit.Text;
  MyDBF.FieldByName('PostCode').AsString := NewEditForm.PostCodeEdit.Text;
  MyDBF.FieldByName('Telephone').AsString := NewEditForm.TelephoneEdit.Text;
  MyDBF.FieldByName('Fax').AsString := NewEditForm.FaxEdit.Text;
  MyDBF.FieldByName('Mobile').AsString := NewEditForm.MobileEdit.Text;
  MyDBF.FieldByName('Email').AsString := NewEDitForm.EmailEdit.Text;
  MyDBF.FieldByName('WWW').AsString := NewEditForm.WebSiteEdit.Text;
  MyDBF.FieldByName('Comments').AsString := NewEditForm.CommentsMemo.Text;
  MyDBF.UpdateRecord;
end;

procedure TContactForm.NewRecData;
begin
  MyDBF.Append;
  MyDBF.FieldByName('Title').AsString := NewEditForm.TitleEdit.Text;
  MyDBF.FieldByName('FirstName').AsString := NewEditForm.FirstNameEdit.Text;
  MyDBF.FieldByName('LastName').AsString := NewEditForm.LastNameEdit.Text;
  MyDBF.FieldByName('Address1').AsString := NewEditForm.Address1Edit.Text;
  MyDBF.FieldByName('Address2').AsString := NewEditform.Address2Edit.Text;
  MyDBF.FieldByName('Town').AsString := NewEditForm.TownEdit.Text;
  MyDBF.FieldByName('County').AsString := NewEditForm.CountyEdit.Text;
  MyDBF.FieldByName('PostCode').AsString := NewEditForm.PostCodeEdit.Text;
  MyDBF.FieldByName('Telephone').AsString := NewEditForm.TelephoneEdit.Text;
  MyDBF.FieldByName('Fax').AsString := NewEditForm.FaxEdit.Text;
  MyDBF.FieldByName('Mobile').AsString := NewEditForm.MobileEdit.Text;
  MyDBF.FieldByName('Email').AsString := NewEDitForm.EmailEdit.Text;
  MyDBF.FieldByName('WWW').AsString := NewEditForm.WebSiteEdit.Text;
  MyDBF.FieldByName('Comments').AsString := NewEditForm.CommentsMemo.Text;
  MyDbf.Post;
  MyDbf.Refresh;
end;

initialization
  {$I mainunit.lrs}

end.
